import React, { useState, useRef, useEffect, useContext } from 'react'
import PropTypes from 'prop-types'
import { usePrevious } from 'react-use'
import { useSpring, animated } from 'react-spring'
import Action from './action.jsx'
import { ThemeContext } from '../utils'

/**
 * Display a set of `options` of which only one can be chosen at once.
 * 
 * @param  {string}   props.selected - Selected option `value`
 * @param  {array}    props.options  - A list of option objects
 * @param  {function} props.onClick  - A callback fired on click
 * @return {object}                  - React markup
 */
const Selector = ({
    selected,
    options,
    onClick
}) => {
    const { neutral, primary } = useContext(ThemeContext)
    const [ position, setPosition ] = useState({ left: -2, width: 0 })
    const prevSelected = usePrevious(selected)
    const ref = useRef(null)
    const style = useSpring({
        width: position.width,
        transform: `translateX(${position.left}px)`
    })

    useEffect(() => {
        const dimensions = ref.current && ref.current.getBoundingClientRect()

        if (dimensions && selected !== prevSelected) {
            setPosition({
                left: ref.current.offsetLeft - 18,
                width: dimensions.width + 18 * 2
            })
        }
    })

    return (
        <Action style={{ position: 'relative' }}>
            <animated.figure style={{
                ...style,
                position: 'absolute',
                top: -2,
                left: 0,
                height: 'calc(100% + 4px)',
                borderRadius: 20,
                boxShadow: `0 0 15px ${neutral[2]}`,
                background: primary[primary.length - 1]
            }} />
            {options.map((option, index) => (
                <span
                    key={option.value}
                    ref={option.value === selected ? ref : undefined}
                    onClick={e => onClick(option, e)}
                    style={{
                        position: 'relative',
                        marginRight: index + 1 < options.length ? 32 : 0,
                        color: option.value === selected ? neutral[0] : neutral[neutral.length - 3],
                        cursor: 'pointer',
                        transition: 'all 250ms'
                    }}
                >
                    {option.label}
                </span>
            ))}
        </Action>
    )
}

Selector.propTypes = {
    selected: PropTypes.string.isRequired,
    options: PropTypes.arrayOf(
        PropTypes.shape({
            label: PropTypes.string.isRequired,
            value: PropTypes.string.isRequired
        })
    ).isRequired,
    onClick: PropTypes.func
}

export default Selector